==> tdf?file=1&path=frealign_v8.11_130826.tar.gz: frealign_v8.11/src/d2mask.f <==
C**************************************************************************
      SUBROUTINE D2MASK(NSAM,B3DV,B3DF,B3DS,XSTD)
C**************************************************************************
C When XSTD is negative (see CALL), produces a sharp (0/1) mask boundary 
C defined by density being above the mean level of a low pass filtered
C map by an amount equal to XSTD times the standard deviation of the map.

C Calls rlft3.
C Used by MASK.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,L,LL,M,MM,N,NN,ID,IS,I,NSAMH,JC,NSAM3
      REAL RAD2,FRAD,B3DV(*),TLEVEL,XSTD,SCAL
      PARAMETER (FRAD=0.1)
      DOUBLE PRECISION DMEAN,DSTD
      COMPLEX B3DF(*),B3DS(*)
C**************************************************************************
      JC=NSAM/2+1
      NSAMH=NSAM/2
      NSAM3=NSAM*NSAM*NSAM
      SCAL=1.0/NSAM/NSAM/NSAMH

      CALL RLFT3(B3DV,B3DS,NSAM,NSAM,NSAM,1)
C
C     Low-pass filter 3D map....
      DO 88 L=1,JC
        LL=L-1
        DO 88 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          DO 88 N=1,NSAM
            NN=N-1
            IF (NN.GE.JC) NN=NN-NSAM
            RAD2=REAL(LL**2+MM**2+NN**2)/NSAM**2/FRAD**2
            IF (L.NE.JC) THEN
              ID=L+NSAMH*((M-1)+NSAM*(N-1))
              B3DF(ID)=B3DF(ID)*EXP(-RAD2)
            ELSE
              IS=M+NSAM*(N-1)
              B3DS(IS)=B3DS(IS)*EXP(-RAD2)
            ENDIF
88    CONTINUE

      CALL RLFT3(B3DV,B3DS,NSAM,NSAM,NSAM,-1)
C
C     Calculate mean, STD
      DSTD=0.0D0
      DMEAN=0.0D0
      DO 87 I=1,NSAM3
        B3DV(I)=B3DV(I)*SCAL
        DMEAN=DMEAN+B3DV(I)
        DSTD=DSTD+B3DV(I)**2
87    CONTINUE
      DMEAN=DMEAN/NSAM3
      DSTD=DSTD/NSAM3
      DSTD=DSTD-DMEAN**2
      IF (DSTD.GT.0.0D0) THEN
        DSTD=SQRT(DSTD)
      ELSE
        DSTD=0.0D0
      ENDIF
C
      WRITE(*,1000) REAL(DMEAN),REAL(DSTD)
1000  FORMAT(' Mean, STD of low-pass filtered map: ',2F14.8)
C
C     Threshold filtered map using XSTD x STD above mean
      ID=0
      TLEVEL=DMEAN+ABS(XSTD)*DSTD
      DO 86 I=1,NSAM3
        IF (B3DV(I).GE.TLEVEL) THEN
          B3DV(I)=1.0
          ID=ID+1
        ELSE
          B3DV(I)=0.0
        ENDIF
86    CONTINUE
C
      WRITE(*,1100) ID
1100  FORMAT(' Number of voxels inside mask:       ',I14,/)
C
      RETURN
      END

==> tdf?file=1&path=frealign_v9.05_131002.tar.gz: frealign_v9.05/src/d2mask.f <==
C*
C* Copyright 2013 Howard Hughes Medical Institute.
C* All rights reserved.
C* Use is subject to Janelia Farm Research Campus Software Copyright 1.1
C* license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
C*
C**************************************************************************
      SUBROUTINE D2MASK(NSAM,B3DV,B3DF,B3DS,XSTD)
C**************************************************************************
C When XSTD is negative (see CALL), produces a sharp (0/1) mask boundary 
C defined by density being above the mean level of a low pass filtered
C map by an amount equal to XSTD times the standard deviation of the map.

C Calls rlft3.
C Used by MASK.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,L,LL,M,MM,N,NN,ID,IS,I,NSAMH,JC,NSAM3
      REAL RAD2,FRAD,B3DV(*),TLEVEL,XSTD
      PARAMETER (FRAD=0.1)
      DOUBLE PRECISION DMEAN,DSTD
      COMPLEX B3DF(*),B3DS(*)
C**************************************************************************
      JC=NSAM/2+1
      NSAMH=NSAM/2
      NSAM3=NSAM*NSAM*NSAM

      CALL RLFT3(B3DV,B3DS,NSAM,NSAM,NSAM,1)
C
C     Low-pass filter 3D map....
      DO 88 L=1,JC
        LL=L-1
        DO 88 M=1,NSAM
          MM=M-1
          IF (MM.GE.JC) MM=MM-NSAM
          DO 88 N=1,NSAM
            NN=N-1
            IF (NN.GE.JC) NN=NN-NSAM
            RAD2=REAL(LL**2+MM**2+NN**2)/NSAM**2/FRAD**2
            IF (L.NE.JC) THEN
              ID=L+NSAMH*((M-1)+NSAM*(N-1))
              B3DF(ID)=B3DF(ID)*EXP(-RAD2)
            ELSE
              IS=M+NSAM*(N-1)
              B3DS(IS)=B3DS(IS)*EXP(-RAD2)
            ENDIF
88    CONTINUE

      CALL RLFT3(B3DV,B3DS,NSAM,NSAM,NSAM,-1)
C
C     Calculate mean, STD
      DSTD=0.0D0
      DMEAN=0.0D0
      DO 87 I=1,NSAM3
        DMEAN=DMEAN+B3DV(I)
        DSTD=DSTD+B3DV(I)**2
87    CONTINUE
      DMEAN=DMEAN/NSAM3
      DSTD=DSTD/NSAM3
      DSTD=DSTD-DMEAN**2
      IF (DSTD.GT.0.0D0) THEN
        DSTD=SQRT(DSTD)
      ELSE
        DSTD=0.0D0
      ENDIF

C     Threshold filtered map using XSTD x STD above mean
      TLEVEL=DMEAN+ABS(XSTD)*DSTD
      DO 86 I=1,NSAM3
        IF (B3DV(I).GE.TLEVEL) THEN
          B3DV(I)=1.0
        ELSE
          B3DV(I)=0.0
        ENDIF
86    CONTINUE

      RETURN
      END
